/*
   rpcclnt.c -- PIKT rpc client (piktc) routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/rpcclnt.c,v 1.51 2000/02/22 03:15:38 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2000 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include "../lib/common.h"

#include <rpc/rpc.h>

#if defined(HAVE_NETCONFIG_H)
#include <netconfig.h>
#endif

#include <rpc/xdr.h>
#include <rpc/clnt.h>
#include "rpc.h"

#include "piktc.h"

CLIENT *clnt;

/* when we recompile this file to generate svc_rpc_clnt.o for linking into
   piktc_svc, we only need the call_back() function, so we put the
   '#ifndef PIKTC_SVC' wrapper around the code we don't need for piktc_svc,
   and the '#else' around the code we do need
 */

#ifndef PIKTC_SVC	/* piktc */
itm command;
itm alert;
#endif

fil _file;		/* piktc & piktc_svc */
req reqserv;		/* piktc & piktc_svc */

const char *transports[] = { "udp", "tcp" };

#ifndef PIKTC_SVC	/* piktc */
int disable_alert(char *, char *, const char *, int);
int enable_alert(char *, char *, char *, int);
int execute_command(char *, char *, char *, int);
int execute_alert(char *, char *, char *, int);
int fetch_file(char *, char *, char *, const char *, int);
int delete_file(char *, char *, char *, const char *, int);
int list_file(char *, char *, char *, const char *, int, int);
int show_alert_status(char *, char *, const char *, int);
#endif

#ifndef PIKTC_SVC	/* piktc */

/* enable all alerts in the supplied list on the given host
 */
void
enable_alerts (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "edit_piktd_conf", inform);
	if (! regres) {
		if (inform) {
			printf("alerts(s) not enabled\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("enabling alert(s)...\n");
		fflush(stdout);
	}
	foreach (np, alts) {
		if (strstr(np->aux, PIKTNEVER))	/* bypass non-alarm scripts */
			continue;
		if (np->state == ERROR) {
			if (inform) {
				printf("%s not enabled, unknown alert\n", np->name);
				fflush(stdout);
			}
			continue;
		}
		if (! (res = list_file(host, np->name, altdir, ".alt", IGNORE, inform)) && (res != IGNORE)) {
			if (inform) {
				printf("%s not enabled, %s.alt not found\n", np->name, np->name);
				fflush(stdout);
			}
			continue;
		}
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("alert(s) not enabled\n");
				fflush(stdout);
			}
			break;
		}
		res = enable_alert(host, np->name, np->aux, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("alert(s) not enabled\n");
				fflush(stdout);
			}
			break;
		}
		if (inform) {
			printf("%s %senabled\n", np->name, res == 1 ? "" : "not ");
			fflush(stdout);
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* disable all alerts in the supplied list on the given host
 */
void
disable_alerts (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "edit_piktd_conf", inform);
	if (! regres) {
		if (inform) {
			printf("alerts(s) not disabled\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("disabling alert(s)...\n");
		fflush(stdout);
	}
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform) {
				printf("%s not enabled, unknown alert\n", np->name);
				fflush(stdout);
			}
			continue;
		}
		res = disable_alert(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("alert(s) not disabled\n");
				fflush(stdout);
			}
			break;
		}
		if (inform) {
			printf("%s %sdisabled\n", np->name, res == 1 ? "" : "not ");
			fflush(stdout);
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* execute all alerts in the supplied list on the given host
 */
void
execute_commands (char *host, nam *cmds, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "execute_command", inform);
	if (! regres) {
		if (inform) {
			printf("command(s) not executed\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("executing command(s)...\n");
		fflush(stdout);
	}
	foreach (np, cmds) {
		if (np->state == ERROR) {
			if (inform) {
				printf("%s not executed, unknown command\n", np->name);
				fflush(stdout);
			}
			continue;
		}
		res = execute_command(host, np->name, np->aux, inform);
		if (((res == NONE) && isexecwait) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("command(s) not executed\n");
				fflush(stdout);
			}
			break;
		}
		if (isexecwait && inform) {
			printf("\"%s\" %sexecuted\n", np->name, res == 1 ? "" : "not ");
			fflush(stdout);
		}
		if (isexecwait) {
			res = free_execcmdout(host, inform);
			if ((res == FALSE) || (res == NONE)) {
				if (inform) {
					printf("remaining command(s) (if any) not executed");
					fflush(stdout);
				}
				break;
			}
		}
	}
	/* if we deregister the execute_command request immediately, a client
	   piktc_svc sending an asynchronous callback might find nothing
	   registered; in the isexec case, we do the deregistration in
	   piktc.c at the conclusion of all host processing
	 */
	if (isexec)
		;	/* wait until all clients are processed */
	else /* isexecwait, or default */
		register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* execute all alerts in the supplied list on the given host
 */
void
execute_alerts (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "execute_alert", inform);
	if (! regres) {
		if (inform) {
			printf("alerts(s)/script(s) not executed\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("executing alert(s)/script(s)...\n");
		fflush(stdout);
	}
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform) {
				printf("%s not executed, unknown alert/script\n", np->name);
				fflush(stdout);
			}
			continue;
		}
		res = execute_alert(host, np->name, np->aux, inform);
		if (((res == NONE) && isexecwait) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("alert(s)/script(s) not executed\n");
				fflush(stdout);
			}
			break;
		}
		if (isexecwait && inform) {
			printf("%s %sexecuted\n", np->name, res == 1 ? "" : "not ");
			fflush(stdout);
		}
	}
	/* if we deregister the execute_alert request immediately, a client
	   piktc_svc sending an asynchronous callback might find nothing
	   registered; in the isexec case, we do the deregistration in
	   piktc.c at the conclusion of all host processing
	 */
	if (isexec)
		;	/* wait until all clients are processed */
	else /* isexecwait, or default */
		register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* install all files in the supplied list on the given host
 */
void
install_files (char *host, nam *fils, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "install_file", inform);
	if (! regres) {
		if (inform) {
			printf("file(s) not installed\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("installing file(s)...\n");
		fflush(stdout);
	}
	foreach (np, fils) {
		res = install_file(host, np->name, dir, ext, mode, uid, gid, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("file(s) not installed\n");
				fflush(stdout);
			}
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			if (inform) {
				printf("%s%s %sinstalled\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				fflush(stdout);
			}
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* fetch all files in the supplied list from the given host
 */
void
fetch_files (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "fetch_file", inform);
	if (! regres) {
		if (inform) {
			printf("file(s) not fetched\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("fetching file(s)...\n");
		fflush(stdout);
	}
	foreach (np, fils) {
		res = fetch_file(host, np->name, dir, ext, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("file(s) not fetched\n");
				fflush(stdout);
			}
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			if (inform) {
				printf("%s%s %sfetched\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				fflush(stdout);
			}
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* delete all files in the supplied list on the given host
 */
void
delete_files (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "delete_file", inform);
	if (! regres) {
		if (inform) {
			printf("file(s) not deleted\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("deleting file(s)...\n");
		fflush(stdout);
	}
	foreach (np, fils) {
		res = delete_file(host, np->name, dir, ext, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("file(s) not deleted\n");
				fflush(stdout);
			}
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			if (inform) {
				printf("%s%s %sdeleted\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				fflush(stdout);
			}
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* list all files in the supplied list on the given host
 */
void
list_files (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "list_file", inform);
	if (! regres) {
		if (inform) {
			printf("file(s) not listed\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("listing file(s)...\n");
		fflush(stdout);
	}
	foreach (np, fils) {
		res = list_file(host, np->name, dir, ext, NIL, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("file(s) not listed\n");
				fflush(stdout);
			}
			break;
		} else if (res == IGNORE) {
			continue;
		} else {
			/* if (inform) */ {
				printf("%s%s %sfound\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				fflush(stdout);
			}
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* kill either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
kill_daemon (char *host, unsigned long service, int inform)
{
	int regres;
	int *res;

	if (service == KILL_PIKTD_SERVICE)
		regres = register_request(host, "kill_piktd", inform);
	else
		regres = register_request(host, "kill_piktc_svc", inform);
	if (! regres) {
		if (inform) {
			printf("daemon not killed\n");
			fflush(stdout);
		}
		exit(1);
	}
	res = (int *) NULL;
	if (inform) {
		printf("killing daemon (%s)...\n", service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc");
		fflush(stdout);
	}
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not killed\n");
			fflush(stdout);
		}
		register_request("", "", FALSE);	/* in effect, deregister */
		return;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "kill_daemon (%s) request to host %s", service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == KILL_PIKTD_SERVICE)
		res = kill_piktd_1((void *) NULL, clnt);
	else
		res = kill_piktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
					printf("daemon not killed\n");
					fflush(stdout);
					break;
				case NONE:
					printf("daemon not running\n");
					fflush(stdout);
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
					printf("daemon not killed\n");
					fflush(stdout);
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon killed\n");
					fflush(stdout);
					break;
				default:
					break;
			}
		} else {
			if (service == KILL_PIKTC_SVC_SERVICE) {
				printf("daemon killed\n");
				fflush(stdout);
			} else {
				fprintf(stderr, "no response from server\n");
				fflush(stderr);
			}
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* restart either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
restart_daemon (char *host, unsigned long service, int inform)
{
	int regres;
	int *res;

	if (service == RESTART_PIKTD_SERVICE)
		regres = register_request(host, "restart_piktd", inform);
	else
		regres = register_request(host, "restart_piktc_svc", inform);
	if (! regres) {
		if (inform) {
			printf("daemon not (re)started\n");
			fflush(stdout);
		}
		exit(1);
	}
	res = (int *) NULL;
	if (inform) {
		printf("(re)starting daemon (%s)...\n", service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc");
		fflush(stdout);
	}
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not (re)started\n");
			fflush(stdout);
		}
		register_request("", "", FALSE);	/* in effect, deregister */
		return;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "restart_daemon (%s) request to host %s", service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == RESTART_PIKTD_SERVICE)
		res = restart_piktd_1((void *) NULL, clnt);
	else
		res = restart_piktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
				case NONE:
					printf("daemon not (re)started\n");
					fflush(stdout);
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
					printf("daemon not (re)started\n");
					fflush(stdout);
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon (re)started\n");
					fflush(stdout);
					break;
				default:
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
			fflush(stderr);
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* report the status of each alert in the supplied list on the given host
 */
void
show_alerts_stata (char *host, nam *alts, int inform)
{
	nam *np;
	int regres;
	int res;

	regres = register_request(host, "show_alert_status", inform);
	if (! regres) {
		if (inform) {
			printf("alert stata not shown\n");
			fflush(stdout);
		}
		exit(1);
	}
	if (inform) {
		printf("showing alert stata...\n");
		fflush(stdout);
	}
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform) {
				printf("%s status not shown, unknown alert\n", np->name);
				fflush(stdout);
			}
			continue;
		}
		res = show_alert_status(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform) {
				printf("alert stata not shown\n");
				fflush(stdout);
			}
			break;
		}
		if (inform) {
			printf("%s %sactive\n", np->name, res == 1 ? "" : "not ");
			fflush(stdout);
		}
	}
	register_request("", "", FALSE);	/* in effect, deregister */
	return;
}

/* report the status of the given alert on the given host
 */
int
show_alert_status (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = (char *) aux;
	alert.state = IGNORE;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "show_alert_status request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = show_alert_status_1((ritm *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				fflush(stderr);
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* enable the given alert on the given host
 */
int
enable_alert (char *host, char *name, char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = ADD;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "edit_piktd_conf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = edit_piktd_conf_1((ritm *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				fflush(stderr);
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* disable the given alert on the given host
 */
int
disable_alert (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = (char *) aux;
	alert.state = DELETE;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "edit_piktd_conf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = edit_piktd_conf_1((ritm *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				fflush(stderr);
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* execute the given alert on the given host
 */
int
execute_command (char *host, char *name, char *aux, int inform)
{
	struct timeval timeout;
	execres *res;

	res = (execres *) NULL;
	command.name = name;
	command.aux = aux;
	if (isexec)
		command.state = EXECUTE;
	else if (isexecwait)
		command.state = EXECUTEWAIT;
	else 
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in execute_command(), isexec or isexecwait not set");
		
	if (isexec)
		clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	else if (isexecwait)
		clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "tcp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	if (isexec)
		timeout.tv_sec = 1L;
	else if (isexecwait)
		timeout.tv_sec = (long) rpc_timeout;
	timeout.tv_usec = 0L;
	if (! clnt_control(clnt, CLSET_TIMEOUT, (char *)&timeout))
		logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_command(), clnt_control() failure to set timeout");
	/* set retry timeout, but for exec/udp only */
	if (isexec && ! istcponly) {
		timeout.tv_sec = 2L;
		timeout.tv_usec = 0L;	/* probably redundant */
		if (! clnt_control(clnt, CLSET_RETRY_TIMEOUT, (char *)&timeout))
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_command(), clnt_control() failure to set retry timeout");
	} /* else if (isexecwait || istcponly)
		is not udp, and setting the retry timout doesn't apply
	   */
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "execute_command request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (isexec)
		res = (execres *) execute_command_1((ritm *) &command, clnt);
	else if (isexecwait)
		res = execute_commandwait_1((ritm *) &command, clnt);
	if (isexecwait) {
		if (res) {
			switch (res->result) {
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				default:
					if (res->execres_u.execcmdout) {
						fprintf(stdout, "%s", res->execres_u.execcmdout);
						fflush(stdout);
					}
					break;
			}
		} else {
			#ifdef AIX
			;
			#else
			fprintf(stderr, "no response from server\n");
			fflush(stderr);
			#endif
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (isexecwait && res) {
		switch (res->result) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else /* if (isexec || ! res) */
		#ifdef AIX
		if (! res)
			return TRUE;
		else
			return NONE;
		#else
		return NONE;
		#endif
}

/* issue a call to free the memory hunk used for temporary storage of
   executed command output
 */
int
free_execcmdout (char *host, int inform)
{
	int *res;

	res = (int *) NULL;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("exec output not cleared\n");
			fflush(stdout);
		}
		return FALSE;
	}
	/* this is imposing too much overhead already!
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "free_execcmdout request to host %s", host);
	*/
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = free_execcmdout_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case 0:
					break;
				default:
					printf("exec output not cleared\n");
					fflush(stdout);
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
			fflush(stderr);
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;

}

/* execute the given alert on the given host
 */
int
execute_alert (char *host, char *name, char *aux, int inform)
{
	struct timeval timeout;
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	if (isexec)
		alert.state = EXECUTE;
	else if (isexecwait)
		alert.state = EXECUTEWAIT;
	else
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in execute_alert(), isexec or isexecwait not set");
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	if (isexec)
		timeout.tv_sec = 1L;
	else if (isexecwait)
		timeout.tv_sec = (long) rpc_timeout;
	timeout.tv_usec = 0L;
	if (! clnt_control(clnt, CLSET_TIMEOUT, (char *)&timeout))
		logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_alert(), clnt_control() failure to set timeout");
	/* whether isexec or isexecwait, and is udp, set the retry timeout */
	if (! istcponly) {	/* hence is udp */
		if (isexec)
			timeout.tv_sec = 2L;
		else if (isexecwait)
			timeout.tv_sec = (long) rpc_retry_timeout;
		timeout.tv_usec = 0L;	/* probably redundant */
		if (! clnt_control(clnt, CLSET_RETRY_TIMEOUT, (char *)&timeout))
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_alert(), clnt_control() failure to set retry timeout");
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "execute_alert request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = execute_alert_1((ritm *) &alert, clnt);
	if (isexecwait) {
		if (res) {
			switch (*res) {
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				default:
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
			fflush(stderr);
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (isexecwait && res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else /* if (isexec || ! res) */
		return NONE;
}

/* fetch the given file from the given host
 */
int
fetch_file (char *host, char *name, char *dir, const char *ext, int inform)
{
	static char _fetch_file_[] = "fetch_file";
	int isdefault;
	static nam *np;
	char *sfn, *tfn;
	struct stat fbuf;
	char cmd[MAXCMD+1];
	int *res;

	res = (int *) NULL;
	isdefault = TRUE;	/* assume the default dir */
	if (! *dir) {	/* signals that this is an other file
			   fetch
			 */
		if (! (np = lookupname(files, name))) {
			if (! filesall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in fetch_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		isdefault = FALSE;
		_file.dir = np->dir;
		_file.file = np->file;
	} else if (! strcmp(ext, "NIL")) {   /* signals that this is a program
				               fetch
				             */
		if (! (np = lookupname(programs, name))) {
			if (! programsall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in fetch_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->dir && *(np->dir)) {
			isdefault = FALSE;
			_file.dir = np->dir;
			_file.file = np->file;
		} else {
			_file.dir = dir;
			_file.file = name;
		}
	} else {			/* go with the defaults */
		_file.dir = dir;
		_file.file = name;
	}
	_file.ext = (char *) ext;
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	if (hostmatch(host, master)) {
		sfn = (char *) mymalloc((strlen(_file.dir)+strlen(_file.file)+strlen(_file.ext)+2) * sizeof(char), _fetch_file_);
		sprintf(sfn, "%s/%s%s", _file.dir, _file.file, strcmp(_file.ext, "NIL") ? _file.ext : "");
		if (stat(sfn, &fbuf)) {
			free(sfn);
			return FALSE;
		} else {
			if (isdefault) {
				tfn = (char *) mymalloc((strlen(difdir)+strlen(_file.file)+strlen(_file.ext)+2) * sizeof(char), _fetch_file_);
				sprintf(tfn, "%s/%s%s", difdir, _file.file, strcmp(_file.ext, "NIL") ? _file.ext : "");
			} else {
				tfn = (char *) mymalloc((strlen(difdir)+strlen(_file.dir)+strlen(_file.file)+strlen(_file.ext)+2) * sizeof(char), _fetch_file_);
				sprintf(tfn, "%s%s/%s%s", difdir, _file.dir, _file.file, strcmp(_file.ext, "NIL") ? _file.ext : "");
			}
			sprintf(cmd, "%s %s %s 2>/dev/null", CPCMD, sfn, tfn);
			system(cmd);
			free(sfn);
			free(tfn);
			return TRUE;
		}
	} else {
		clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
		if (clnt == NULL) {
			if (inform)
				clnt_pcreateerror(host);
			return NONE;
		}
		logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "fetch_file request to host %s", host);
		(AUTH *) clnt->cl_auth = authunix_create_default();
		res = fetch_file_1((rfil *) &_file, clnt);
		if (res) {
			switch (*res) {
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				default:
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
			fflush(stderr);
		}
		/* auth_destroy(clnt->cl_auth); */
		clnt_destroy(clnt);
		if (res) {
			switch (*res) {
				case 0:
					return TRUE;
					break;
				case UNAUTHORIZED:
					return UNAUTHORIZED;
					break;
				default:
					return FALSE;
					break;
			}
		} else
			return NONE;
	}
}

/* delete the given file on the given host
 */
int
delete_file (char *host, char *name, char *dir, const char *ext, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	if (! *dir) {	/* signals that this is an other file
			   deletion, in which case override
			   the default dir
			 */
		if (! (np = lookupname(files, name))) {
			if (! filesall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in delete_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.dir = np->dir;
		_file.file = np->file;
	} else if (! strcmp(ext, "NIL")) {   /* signals that this is a program
				               deletion, in which case possibly
				               override the default dir
				             */
		if (! (np = lookupname(programs, name))) {
			if (! programsall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in delete_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->dir && *(np->dir)) {
			_file.dir = np->dir;
			_file.file = np->file;
		} else {
			_file.dir = dir;
			_file.file = name;
		}
	} else {
		_file.dir = dir;
		_file.file = name;
	}
	_file.ext = (char *) ext;
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "delete_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = delete_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				fflush(stderr);
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* report if the given file exists on the given host
 */
int
list_file (char *host, char *name, char *dir, const char *ext, int status, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	if (! *dir) {	/* signals that this is an other file
			   lookup, in which case override
			   the default dir
			 */
		if (! (np = lookupname(files, name))) {
			if (! filesall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in list_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.dir = np->dir;
		_file.file = np->file;
	} else if (! strcmp(ext, "NIL")) {   /* signals that this is a program
				               lookup, in which case possibly
				               override the default dir
				             */
		if (! (np = lookupname(programs, name))) {
			if (! programsall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in list_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->dir && *(np->dir)) {
			_file.dir = np->dir;
			_file.file = np->file;
		} else {
			_file.dir = dir;
			_file.file = name;
		}
	} else {
		_file.dir = dir;
		_file.file = name;
	}
	_file.ext = (char *) ext;
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "list_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = list_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				if (status != IGNORE) {
					fprintf(stderr, "service unauthorized\n");
					fflush(stderr);
				}
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				if (status == IGNORE)
					return IGNORE;
				else
					return UNAUTHORIZED;
				break;
			default:
				if (status == IGNORE)
					return IGNORE;
				else
					return FALSE;
				break;
		}
	} else
		return NONE;
}

/* register a service request with the local piktc_svc
 */
int
register_request (char *host, char *request, int inform)
{
	struct timeval timeout;
	int *res;

	res = (int *) NULL;
	reqserv.hostname = host;
	reqserv.request = request;
	if (! *request) {	/* request is "", i.e., a deregistration */
		if (isexec)
			reqserv.state = EXECUTE;	/* leave in place for
							   possible late
							   callbackers
							 */
		else
			reqserv.state = NONE;
	} else {
		if (isexec)
			reqserv.state = EXECUTE;
		else if (isexecwait)
			reqserv.state = EXECUTEWAIT;
	}
	/* hostname is the name of the local host--i.e., the machine
	   where the piktc service request is issuing from
	 */
	clnt = clnt_create(hostname, PIKTCPROG, PIKTCVERS, transports[istcponly]);
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(hostname);
		return NONE;
	}
	/* if the master piktc_svc is busy handling many incoming callback
	   requests for an exec (no wait), we need to up the timeout; else
	   go with the default
	 */
	if (isexec) {
		timeout.tv_sec = (long) rpc_timeout;
		timeout.tv_usec = 0L;
		if (! clnt_control(clnt, CLSET_TIMEOUT, (char *)&timeout))
			logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_alert(), clnt_control() failure to set timeout");
		if (! istcponly) {	/* hence is udp */
			timeout.tv_sec = (long) rpc_retry_timeout;
			timeout.tv_usec = 0L;	/* probably redundant */
			if (! clnt_control(clnt, CLSET_RETRY_TIMEOUT, (char *)&timeout))
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, "in execute_alert(), clnt_control() failure to set retry timeout");
		}
	}
	if (isverboselog)
		logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "register_request request to host %s", hostname);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = register_request_1((rqst *) &reqserv, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case UNAUTHORIZED:
					fprintf(stderr, "register service unauthorized\n");
					fflush(stderr);
					break;
				case DENIED:
					fprintf(stderr, "local access denied\n");
					fflush(stderr);
					remove(piktclock);
					exit(1);
					break;
				default:	/* should only be 0, i.e., success */
					break;
			}
		} else {
			fprintf(stderr, "no response from local server\n");
			fflush(stderr);
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return FALSE;
}

#else	/* #ifndef PIKTC_SVC */		/* piktc_svc */

/* call back to the master piktc_svc, to see if this hostname and service
   request are registered in that piktc_svc
 */
int
call_back (char *host, const char *request, int inform)
{

#ifndef NO_RPC_SVC

	/* for systems supporting the call_back feature */

	int *res;

	if (IS_CALL_BACK) {

		res = (int *) NULL;
		reqserv.hostname = host;
		reqserv.request = (char *) request;
		reqserv.state = IGNORE;

		clnt = clnt_create(master, PIKTCPROG, PIKTCVERS, transports[istcponly]);
		if (clnt == NULL) {
			/*
			clnt_pcreateerror(master);
			*/
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in call_back(), clnt_pcreateerror() for host %s", master);
			return DENIED;
		}
		if (isverboselog)
			logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "call_back request to host %s", master);
		(AUTH *) clnt->cl_auth = authunix_create_default();
		res = call_back_1((rqst *) &reqserv, clnt);
		if (res) {
			switch (*res) {
				case DENIED:
					logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in chkrequest(), %s request not registered on master %s", request, master);
					break;
				default:	/* should only be 0, i.e., success */
					if (isverboselog)
						logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "%s request registered on master %s", request, master);
					break;
			}
		} else
			logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in call_back(), no response from master server\n");
		/* auth_destroy(clnt->cl_auth); */
		clnt_destroy(clnt);
		if (res) {
			switch (*res) {
				case DENIED:
					return DENIED;
					break;
				default:
					return 0;
					break;
			}
		} else
			return DENIED;

	} else {

		return 0;	/* if call_back has been turned off,
				   always return success; note that if
				   call_back has been turned off, we
				   should never enter this function anyway
				 */

	}

#else	/* #ifndef NO_RPC_SVC */

	/* function stub for systems not supporting call_back and where
	   NO_RPC_SVC is #define'd; this is required so that the
	   svc_rpc_clnt.c compile doesn't come up empty-handed (i.e.,
	   that there is a resulting svc_rpc_clnt.o for linking into
	   piktc_svc)
	 */

	return 0;

#endif	/* #ifndef NO_RPC_SVC */

}

#endif	/* #ifndef PIKTC_SVC */

/* install_file() is in both piktc & piktc_svc */

/* install the given file on the given host
 */
int
install_file (char *host, char *name, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	static char _install_file_[] = "install_file";
	int isdefault;
	static nam *np;
	static char *fn;
	#ifdef PIKTC_SVC	/* piktc_svc */
	static char *dn;
	#endif
	struct stat fbuf;
	off_t size;
	FILE *fp;
	int *res;

	res = (int *) NULL;
	isdefault = TRUE;	/* assume the default dir */
	#ifndef PIKTC_SVC	/* piktc */
	if (! *dir) {	/* signals that this is an other file
			   install, in which case override
			   the default dir, mode, uid & gid
			 */
		if (! (np = lookupname(files, name))) {
			if (! filesall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in install_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		isdefault = FALSE;
		_file.dir = np->dir;
		_file.file = np->file;
		_file.ext = (char *) ext;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else if (! strcmp(ext, "NIL")) {   /* signals that this is a program
				               install, in which case possibly
				               override the default dir, mode,
				               uid & gid
				             */
		if (! (np = lookupname(programs, name))) {
			if (! programsall) {
				logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in install_file(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->dir && *(np->dir)) {
			isdefault = FALSE;
			_file.dir = np->dir;
			_file.file = np->file;
		} else {
			_file.dir = dir;
			_file.file = name;
		}
		_file.ext = (char *) ext;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else {			/* go with the defaults */
		_file.dir = dir;
		_file.file = name;
		_file.ext = (char *) ext;
		_file.mode = mode;
		_file.uid = uid;
		_file.gid = gid;
	}
	if (isdefault) {
		fn = (char *) mymalloc((strlen(stgdir)+strlen(_file.file)+strlen(_file.ext)+2) * sizeof(char), _install_file_);
		sprintf(fn, "%s/%s%s", stgdir, _file.file, strcmp(_file.ext, "NIL") ? _file.ext : "");
	} else {
		fn = (char *) mymalloc((strlen(stgdir)+strlen(_file.dir)+strlen(_file.file)+strlen(_file.ext)+2) * sizeof(char), _install_file_);
		sprintf(fn, "%s%s/%s%s", stgdir, _file.dir, _file.file, strcmp(_file.ext, "NIL") ? _file.ext : "");
	}
	#else			/* piktc_svc */
	#ifndef NO_RPC_SVC
	if (! *ext || (! strcmp(ext, "NIL") && strcmp(dir, prgdir)))
		isdefault = FALSE;
	if (isdefault) {
		dn = mystrdup(difdir, _install_file_);
	} else {
		dn = (char *) mymalloc((strlen(difdir)+strlen(dir)+1) * sizeof(char), _install_file_);
		sprintf(dn, "%s%s", difdir, dir);
	}
	_file.dir = dn;
	_file.file = name;
	_file.ext = (char *) ext;
	_file.mode = mode;
	_file.uid = uid;
	_file.gid = gid;
	fn = (char *) mymalloc((strlen(dir)+strlen(name)+strlen(ext)+2) * sizeof(char), _install_file_);
	sprintf(fn, "%s/%s%s", dir, name, strcmp(ext, "NIL") ? ext : "");
	#else	/* #ifndef NO_RPC_SVC */
	return 0;	/* piktc_svc-to-piktc_svc connections not supported */
	#endif	/* #ifndef NO_RPC_SVC */
	#endif	/* #ifndef PIKTC_SVC */
	if (stat(fn, &fbuf)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in install_file(), stat() failure for %s", fn);
		free(fn);
		return 0;
	}
	size = fbuf.st_size;
	_file.size = size;
	_file.text = (char *) mymalloc((size_t)(size+1)*sizeof(char), _install_file_);
	if ((fp = fopen(fn, "r")) == NULL) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in install_file(), fopen() error for %s", fn);
		free(fn);
		return 0;
	}
	fread(_file.text, sizeof(char), (size_t)size, fp);
	if (fclose(fp)) {
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in install_file(), fclose() error for %s", fn);
		free(fn);
		return 0;
	}
	free(fn);
	*(_file.text+size) = '\0';
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "tcp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, LOG_INFO, __FILE__, __LINE__, FALSE, "install_file request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = install_file_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				fflush(stderr);
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				fflush(stderr);
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else {
		#ifdef AIX
		;
		#else
		fprintf(stderr, "no response from server\n");
		fflush(stderr);
		#endif
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	FREE(_file.text);
	#ifdef PIKTC_SVC	/* piktc_svc */
	#ifndef NO_RPC_SVC
	FREE(dn);
	#endif
	#endif
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		#ifdef AIX
		return TRUE;
		#else
		return NONE;
		#endif
}
